@@ -73,38 +73,38 @@ Crypt::Sodium::XS::OO::auth - Secret key message authentication
   my $key = $auth->keygen;
   my $msg = "authenticate this message";
 
-  my $mac = $auth->auth($msg, $key);
-  die "message tampered with!" unless $auth->verify($mac, $msg, $key);
+  my $tag = $auth->auth($msg, $key);
+  die "message tampered with!" unless $auth->verify($tag, $msg, $key);
 
   my $multipart = $auth->init($key);
   $multipart->update("authenticate");
   $multipart->update(" this", " message");
-  $mac = $multipart->final;
-  die "message tampered with!" unless $auth->verify($mac, $msg, $key);
+  $tag = $multipart->final;
+  die "message tampered with!" unless $auth->verify($tag, $msg, $key);
 
 =head1 DESCRIPTION
 
-L<Crypt::Sodium::XS::OO::auth> computes an authentication MAC for a message and
-a secret key, and provides a way to verify that a given MAC is valid for a
+L<Crypt::Sodium::XS::OO::auth> computes an authentication tag for a message and
+a secret key, and provides a way to verify that a given tag is valid for a
 given message and a key.
 
-The function computing the MAC is deterministic: the same C<($message, $key)>
+The function computing the tag is deterministic: the same C<($message, $key)>
 tuple will always produce the same output. However, even if the message is
-public, knowing the key is required in order to be able to compute a valid MAC.
-Therefore, the key should remain confidential. The MAC, however, can be public.
+public, knowing the key is required in order to be able to compute a valid tag.
+Therefore, the key should remain confidential. The tag, however, can be public.
 
 A typical use case is:
 
-* Alice prepares a message, adds an authentication MAC, sends it to Bob
+* Alice prepares a message, adds an authentication tag, sends it to Bob
 
 * Alice doesn't store the message
 
-* Later on, Bob sends the message and the authentication MAC back to Alice
+* Later on, Bob sends the message and the authentication tag back to Alice
 
-* Alice uses the authentication MAC to verify that she created this message
+* Alice uses the authentication tag to verify that she created this message
 
 L<Crypt::Sodium::XS::OO::auth> does not encrypt the message. It only computes
-and verifies an authentication MAC.
+and verifies an authentication tag.
 
 =head1 CONSTRUCTOR
 
@@ -121,58 +121,103 @@ primitive is C<default>.
 
 =head2 primitive
 
-  my $primitive = $aead->primitive;
-  $aead->primitive('chacha20poly1305');
+  my $primitive = $auth->primitive;
+  $auth->primitive('hmacsha256');
 
-The primitive used for all operations by this object.
+Gets or sets the primitive used for all operations by this object. Note this
+can be C<default>.
 
 =head1 METHODS
 
+=head2 primitives
+
+  my @primitives = Crypt::Sodium::XS::OO::auth->primitives;
+  my @primitives = $auth->primitives;
+
+Returns a list of all supported primitive names, including C<default>.
+
+Can be called as a class method.
+
 =head2 PRIMITIVE
 
   my $primitive = $auth->PRIMITIVE;
 
-=head2 BYTES
-
-  my $mac_length = $auth->BYTES;
-
-=head2 KEYBYTES
-
-  my $key_length = $auth->KEYBTES;
-
-=head2 primitives
-
-  my @primitives = $auth->primities;
-
-Returns a list of all supported primitive names (including 'default').
+Returns the primitive used for all operations by this object. Note this will
+never be C<default> but would instead be the primitive it represents.
 
 =head2 auth
 
-  my $mac = $auth->auth($message, $key);
+  my $tag = $auth->auth($message, $key);
+
+C<$message> is the message to authenticate. It may be a
+L<Crypt::Sodium::XS::MemVault>.
+
+C<$key> is the secret key with which to generate the authentication tag. It
+must be L</KEYBYTES> bytes. It may be a L<Crypt::Sodium::XS::MemVault>.
+
+Returns the authentication tag. The tag is L</BYTES> bytes.
 
 =head2 init
 
-  my $multipart = $auth->init($key);
+  my $multipart = $auth->init($key, $flags);
 
-See L</MULTI-PART INTERFACE>.
+C<$key> is the secret key used by the multipart object. It should be at least
+L</KEYBYTES> bytes. It may be a L<Crypt::Sodium::XS::MemVault>.
+
+C<$flags> is optional. It is the flags used for the multipart protected memory
+object. See L<Crypt::Sodium::XS::ProtMem>.
+
+Returns an opaque protected memory object: a multi-part auth object. This is
+useful when authenticating a stream or large message in chunks, rather than in
+one message. See L</MULTI-PART INTERFACE>.
+
+B<Note>: The multipart interface may use arbitrary-size keys. This is not
+recommended as it can be easily misused (e.g., accidentally using an empty
+key). Avoid by always using keys of L</KEYBYTES> bytes as returned by
+L</keygen>.
 
 =head2 keygen
 
-  my $key = $auth->keygen;
+  my $key = $auth->keygen($flags);
+
+C<$flags> is optional. It is the flags used for the C<$key>
+L<Crypt::Sodium::XS::MemVault>. See L<Crypt::Sodium::XS::ProtMem>.
+
+Returns a L<Crypt::Sodium::XS::MemVault>: a new secret key of L</KEYBYTES>
+bytes.
 
 =head2 verify
 
-  my $is_valid = $auth->verify($mac, $message, $key);
+  my $is_valid = $auth->verify($tag, $message, $key);
+
+C<$tag> is the previously generated authentication tag. It must be L</BYTES>
+bytes.
+
+C<$message> is the message to authenticate.
+
+C<$key> is the secret key used to generate the authentication tag. It must be
+L</KEYBYTES> bytes. It may be a L<Crypt::Sodium::XS::MemVault>.
+
+Returns true if C<$tag> is a valid tag for C<$message> and C<$key>, false
+otherwise.
+
+=head2 BYTES
+
+  my $tag_size = $auth->BYTES;
+
+The size, in bytes, of an authentication tag.
+
+=head2 KEYBYTES
+
+  my $key_size = $auth->KEYBYTES;
+
+The size, in bytes, of a secret key.
 
 =head1 MULTI-PART INTERFACE
 
-NOTE: The multipart interface may use arbitrary-length keys. This is not
-recommended as it can be easily misused (e.g., accidentally using an empty
-key).
-
 A multipart auth object is created by calling the L</init> method. Data to be
 authenticated is added by calling the L</update> method of that object as many
-times as desired. An output mac is generated by calling its L</final> method.
+times as desired. An output tag is generated by calling its L</final> method.
 Do not use the object after calling L</final>.
 
 The multipart auth object is an opaque object which provides the following
@@ -187,16 +232,19 @@ state.
 
 =head2 final
 
-  my $mac = $multipart->final;
+  my $tag = $multipart->final;
 
-Once C<final> has been called, the auth object must not be used further.
+Returns a tag for C<$key> (from L</init>) and all authenticated data (from
+L</update>). The tag is L</BYTES> bytes.
+
+Once L</final> has been called, the multipart object must not be used further.
 
 =head2 update
 
-  $multipart->update($message);
   $multipart->update(@messages);
 
-Adds all given arguments (stringified) to authenticated data.
+Adds all given arguments (stringified) to authenticated data. Any argument may
+be a L<Crypt::Sodium::XS::MemVault>.
 
 =head1 SEE ALSO
 
