@@ -47,53 +47,60 @@ Crypt::Sodium::XS::kdf - Secret subkey derivation from a main secret key
   use Crypt::Sodium::XS::kdf ":default";
 
   my $context = "see notes below about context strings";
-  my $output_key_length = 32;
+  my $output_key_size = 32;
   my $master_key = kdf_keygen();
-  my $subkey_1 = kdf_derive($master_key, 1, $output_key_length, $context);
-  my $subkey_2 = kdf_derive($master_key, 2, $output_key_length, $context);
-  my $subkey_3 = kdf_derive($master_key, 54321, $output_key_length, $context);
+  my $subkey_1 = kdf_derive($master_key, 1, $output_key_size, $context);
+  my $subkey_2 = kdf_derive($master_key, 2, $output_key_size, $context);
+  my $subkey_3 = kdf_derive($master_key, 54321, $output_key_size, $context);
 
 =head1 DESCRIPTION
 
-NOTE: Secret keys used to encrypt or sign confidential data have to be chosen
-from a very large keyspace. However, passwords are usually short,
-human-generated strings, making dictionary attacks practical. If you are
-intending to derive keys from a password, see L<Crypt::Sodium::XS::pwhash>
-instead.
-
 Multiple secret subkeys can be derived from a single high-entropy master key.
 Given the master key and a numeric key identifier, a subkey can be
 deterministically computed. However, given a subkey, an attacker cannot compute
 the master key nor any other subkeys.
 
+B<Note>: Secret keys used to encrypt or sign confidential data have to be
+chosen from a very large keyspace. However, passwords are usually short,
+human-generated strings, making dictionary attacks practical. If you are
+intending to derive keys from a password, see L<Crypt::Sodium::XS::pwhash>
+instead.
+
 =head1 FUNCTIONS
 
 Nothing is exported by default. A C<:default> tag imports the functions and
-constants as documented below. A separate import tag is provided for each of
-the primitives listed in L</PRIMITIVES>. For example, C<:blake2b> imports
-C<kdf_blake2b_derive>. You should use at least one import tag.
+constants documented below. A separate C<:E<lt>primitiveE<gt>> import tag is
+provided for each of the primitives listed in L</PRIMITIVES>. These tags import
+the C<kdf_E<lt>primitiveE<gt>_*> functions and constants for that primitive. A
+C<:all> tag imports everything.
 
 =head2 kdf_derive
 
-  my $subkey = kdf_derive($key, $id, $subkey_len, $context);
+=head2 kdf_E<lt>primitiveE<gt>_derive
 
-C<$key> is the master key from which others should be derived.
+  my $subkey = kdf_derive($key, $id, $subkey_size, $context, $flags);
+
+C<$key> is the master key from which others should be derived. It must be
+L</kdf_KEYBYTES> bytes. It may be a L<Crypt::Sodium::XS::MemVault>.
 
 C<$id> is an unsigned integer signifying the numeric identifier of the subkey
-which is being derived. The same C<$key>, C<$id>, C<$length>, and C<$context>
-will always derive the same key.
+which is being derived. The same C<$key>, C<$id>, C<$subkey_size>, and
+C<$context> will always derive the same key.
 
-C<$subkey_len> is the desired length of the subkey output. This can be used to
-derive a key of the particular length needed for the primitive with which the
-subkey will be used. It must be between L</kdf_BYTES_MIN> and L</kdf_BYTES_MAX>
-(inclusive).
+C<$subkey_size> is the size, in bytes, of the subkey output. This can be used
+to derive a key of the particular size needed for the primitive with which the
+subkey will be used. It must be in the range of L</kdf_BYTES_MIN> to
+L</kdf_BYTES_MAX>, inclusive.
 
-C<$context> as an arbitrary string which is at least L</kdf_CONTEXTBYTES> (see
-warning below). This can be used to create an application-specific tag, such
-that using the same C<$key>, C<$id>, and C<$subkey_len> can still derive a
-different subkey.
+C<$context> is optional. It is an arbitrary string which is at least
+L</kdf_CONTEXTBYTES> bytes (see warning below). This can be used to create an
+application-specific tag, such that using the same C<$key>, C<$id>, and
+C<$subkey_size> can still derive a different subkey.
 
-B<WARNING>: C<$context> must be at least L</kdf_CONTEXTBYTES> in length. If it
+C<$flags> is optional. It is the flags used for the C<$subkey>
+L<Crypt::Sodium::XS::MemVault>. See L<Crypt::Sodium::XS::ProtMem>.
+
+B<WARNING>: C<$context> must be at least L</kdf_CONTEXTBYTES> bytes. If it
 is longer than this, only the first L</kdf_CONTEXTBYTES> bytes will be used. As
 this gives a limited range of use (application-specific strings might be likely
 to have the same first 8 bytes), it is recommended to use an arbitrary-length
@@ -104,7 +111,14 @@ C<$context>.
 
 =head2 kdf_keygen
 
-  my $key = kdf_keygen();
+=head2 kdf_E<lt>primitiveE<gt>_keygen
+
+  my $key = kdf_keygen($flags);
+
+C<$flags> is optional. It is the flags used for the C<$key>
+L<Crypt::Sodium::XS::MemVault>. See L<Crypt::Sodium::XS::ProtMem>.
+
+Returns a L<Crypt::Sodium::XS::MemVault>: a master key of L</KEYBYTES> bytes.
 
 =head1 CONSTANTS
 
@@ -112,21 +126,31 @@ C<$context>.
 
   my $default_primitive = kdf_PRIMITIVE();
 
+Returns the name of the default primitive.
+
 =head2 kdf_BYTES_MAX
 
-  my $subkey_max_length = kdf_BYTES_MAX();
+=head2 kdf_E<lt>primitiveE<gt>_BYTES_MAX
+
+  my $subkey_max_size = kdf_BYTES_MAX();
 
 =head2 kdf_BYTES_MIN
 
-  my $subkey_min_length = kdf_BYTES_MIN();
+=head2 kdf_E<lt>primitiveE<gt>_BYTES_MIN
+
+  my $subkey_min_size = kdf_BYTES_MIN();
 
 =head2 kdf_CONTEXTBYTES
 
-  my $context_length = kdf_CONTEXTBYTES();
+=head2 kdf_E<lt>primitiveE<gt>_CONTEXTBYTES
+
+  my $context_size = kdf_CONTEXTBYTES();
 
 =head2 kdf_KEYBYTES
 
-  my $main_key_length = kdf_KEYBYTES();
+=head2 kdf_E<lt>primitiveE<gt>_KEYBYTES
+
+  my $main_key_size = kdf_KEYBYTES();
 
 =head1 PRIMITIVES
 
@@ -136,7 +160,7 @@ kdf_blake2b_BYTES_MIN).
 
 =over 4
 
-=item * blake2b
+=item * blake2b (default)
 
 =back
 
